@@ -784,6 +784,8 @@ sub tests {
 sub run_cc_test {
     my ($cnt, $backend, $script, $expect, $keep_c, $keep_c_fail, $todo) = @_;
     my ($opt, $got);
+    local($\, $,);   # guard against -l and other things that screw with
+                     # print
     $expect =~ s/\n$//;
     my $fnbackend = lc($backend); #C,-O2
     ($fnbackend,$opt) = $fnbackend =~ /^(cc?)(,-o.)?/;
@@ -824,8 +826,8 @@ sub run_cc_test {
         system($cmdline);
         unless (-e $exe) {
             print "not ok $cnt $todo failed $cmdline\n";
-            print STDERR "# ",system("$Config{cc} $command");
-            unlink ($test, $cfile, $exe, @obj) if !$keep_c;
+            print STDERR "# ",system("$Config{cc} $command"), "\n";
+            #unlink ($test, $cfile, $exe, @obj) unless $keep_c_fail;
             return 0;
         }
         $exe = "./".$exe unless $^O eq 'MSWin32';
@@ -836,12 +838,11 @@ sub run_cc_test {
             }
             if ($got =~ /^$expect$/) {
                 print "ok $cnt", $todo eq '#' ? "\n" : " $todo\n";
-                unlink ($test, $cfile, $exe, @obj) if !$keep_c and ! -s $cfile;
+                unlink ($test, $cfile, $exe, @obj) unless $keep_c;
                 return 1;
             } else {
-                $keep_c = $keep_c_fail unless $keep_c;
                 print "not ok $cnt $todo wanted: \"$expect\", got: \"$got\"\n";
-                unlink ($test, $cfile, $exe, @obj) if !$keep_c and ! -s $cfile;
+                unlink ($test, $cfile, $exe, @obj) unless $keep_c_fail;
                 return 0;
             }
         } else {
@@ -849,8 +850,7 @@ sub run_cc_test {
         }
     }
     print "not ok $cnt $todo wanted: \"$expect\", \$\? = $?, got: \"$got\"\n";
-    $keep_c = $keep_c_fail unless $keep_c;
-    unlink ($test, $cfile, $exe, @obj) if !$keep_c;
+    unlink ($test, $cfile, $exe, @obj) unless $keep_c_fail;
     return 0;
 }
 
@@ -880,8 +880,8 @@ sub run_c_tests {
 
     use Config;
     my $AUTHOR      = -d ".svn";
-    my $keep_c      = 0;	# set it to keep the pl, c and exe files
-    my $keep_c_fail = $AUTHOR;	# set it to keep the pl, c and exe files on failures
+    my $keep_c      = 0;	  # set it to keep the pl, c and exe files
+    my $keep_c_fail = 1;          # keep on failures
 
     my %todo = map { $_ => 1 } @todo;
     my %skip = map { $_ => 1 } @skip;
